@@ -32,6 +32,8 @@ parse_conv_ranef = function(x, df){
       Vphy = Vphy[spl, spl] # same order as species levels
     }
     
+    if(inherits(xx, "dist")) xx = as.matrix(xx)
+    
     if(inherits(xx, c("matrix", "Matrix"))){
       # already a cov matrix
       if(length(setdiff(spl, row.names(xx)))) 
@@ -798,32 +800,42 @@ pglmm.V <- function(par, Zt, St, mu, nested, family, size) {
 }
 # End pglmm.V
 
+#' Testing statistical significance of random effect
+#' 
 #' \code{pglmm_profile_LRT} tests statistical significance of the 
-#' phylogenetic random effect of binomial models on 
-#' species slopes using a likelihood ratio test.
+#' phylogenetic random effect using profile likelihoods when bayes = F. 
+#' The resulting p-values are conditional on the fixed 
+#' estimates of the other parameters, in contrast to a standard 
+#' likelihood ratio test.
 #' 
 #' @rdname pglmm-profile-LRT
-#' @param x A fitted model with class communityPGLMM and family "binomial".
-#' @param re.number Which random term to test? Can be a vector with length >1
+#' @param x A fitted model with class communityPGLMM with `bayes = FALSE`.
+#' @param re.number Which random term to test? Can be a vector with length >1.
 #' @inheritParams pglmm
 #' @return A list of likelihood, df, and p-value.
 #' @export
 #' 
 pglmm_profile_LRT <- function(x, re.number = 0, cpp = TRUE) {
+  if(x$bayes) stop("pglmm_profile_LRT only works with models fitted with `bayes = FALSE`")
   n <- dim(x$X)[1]
   p <- dim(x$X)[2]
   par <- x$ss
   par[re.number] <- 0
   df <- length(re.number)
   
-  if(cpp){
-    LL <- pglmm_LL_cpp(par = x$ss, H = x$H, X = x$X, Zt = x$Zt, St = x$St, 
-                       mu = x$mu, nested = x$nested, REML = x$REML, verbose = FALSE,
-                       family = x$family, totalSize = x$size)
+  if (x$family == "gaussian") {
+    LL <- pglmm_gaussian_LL_calc(par = x$ss, X = x$X, Y = x$Y, Zt = x$Zt, 
+            St = x$St, nested = x$nested, REML = x$REML, verbose = F, optim_ll = TRUE)
   } else {
-    LL <- pglmm.LL(par = x$ss, H = x$H, X = x$X, Zt = x$Zt, St = x$St, 
-                   mu = x$mu, nested = x$nested, REML = x$REML, verbose = FALSE, 
-                   family = x$family, size = x$size)
+    if (cpp) {
+      LL <- pglmm_LL_cpp(par = x$ss, H = x$H, X = x$X, Zt = x$Zt, 
+             St = x$St, mu = x$mu, nested = x$nested, REML = x$REML, verbose = FALSE, 
+              family = x$family, totalSize = x$size)
+    } else {
+      LL <- pglmm.LL(par = x$ss, H = x$H, X = x$X, Zt = x$Zt, 
+              St = x$St, mu = x$mu, nested = x$nested, REML = x$REML, verbose = FALSE,  
+              family = x$family, size = x$size)
+    }
   }
   
   if (x$REML) {
@@ -832,14 +844,19 @@ pglmm_profile_LRT <- function(x, re.number = 0, cpp = TRUE) {
     logLik <- -0.5 * n * log(2 * pi) - LL
   }
   
-  if(cpp){
-    LL0 <- pglmm_LL_cpp(par = par, H = x$H, X = x$X, Zt = x$Zt, St = x$St, 
-                        mu = x$mu, nested = x$nested, REML = x$REML, verbose = FALSE,
-                        family = x$family, totalSize = x$size)
+  if (x$family == "gaussian") {
+    LL0 <- pglmm_gaussian_LL_calc(par = par, X = x$X, Y = x$Y, Zt = x$Zt, 
+              St = x$St, nested = x$nested, REML = x$REML, verbose = F, optim_ll = TRUE)
   } else {
-    LL0 <- pglmm.LL(par = par, H = x$H, X = x$X, Zt = x$Zt, St = x$St, 
-                    mu = x$mu, nested = x$nested, REML = x$REML, verbose = FALSE, 
-                    family = x$family, size = x$size)
+    if (cpp) {
+      LL0 <- pglmm_LL_cpp(par = par, H = x$H, X = x$X, Zt = x$Zt, St = x$St, 
+              mu = x$mu, nested = x$nested, REML = x$REML, verbose = FALSE, 
+              family = x$family, totalSize = x$size)
+    } else {
+      LL0 <- pglmm.LL(par = par, H = x$H, X = x$X, Zt = x$Zt, St = x$St, 
+              mu = x$mu, nested = x$nested, REML = x$REML, verbose = FALSE, 
+              family = x$family, size = x$size)
+    }
   }
   
   if (x$REML) {
@@ -849,11 +866,13 @@ pglmm_profile_LRT <- function(x, re.number = 0, cpp = TRUE) {
   }
   
   P.H0.s2 <- pchisq(2 * (logLik - logLik0), df = df, lower.tail = FALSE)/2
-  if (P.H0.s2 > 0.499) P.H0.s2 <- 1
+  if (P.H0.s2 > 0.499) 
+    P.H0.s2 <- 1
   
   list(LR = logLik - logLik0, df = df, Pr = P.H0.s2)
 }
 
+
 #' @export
 #' @rdname pglmm-profile-LRT
 #' @inheritParams pglmm_profile_LRT
@@ -862,6 +881,7 @@ communityPGLMM.profile.LRT = function(x, re.number = 0, cpp = TRUE){
   pglmm_profile_LRT(x, re.number, cpp)
 }
 
+
 #' \code{pglmm_matrix_structure} produces the entire
 #' covariance matrix structure (V) when you specify random effects.
 #' @param ss Which of the \code{random.effects} to produce.
@@ -965,16 +985,14 @@ summary.communityPGLMM <- function(object, digits = max(3, getOption("digits") -
                 "WAIC" = unname(x$WAIC))
     print(logLik, digits = digits)
   } else {
-    if (x$family == "gaussian") {
-      logLik = x$logLik
-      AIC = x$AIC
-      BIC = x$BIC
-      
-      names(logLik) = "logLik"
-      names(AIC) = "AIC"
-      names(BIC) = "BIC"
-      print(c(logLik, AIC, BIC), digits = digits)
-    }
+    logLik = x$logLik
+    AIC = x$AIC
+    BIC = x$BIC
+    
+    names(logLik) = "logLik"
+    names(AIC) = "AIC"
+    names(BIC) = "BIC"
+    print(c(logLik, AIC, BIC), digits = digits)
   }
   
   if(grepl("zeroinflated", x$family)) {
@@ -1334,18 +1352,24 @@ predict.communityPGLMM <- function(object, newdata = NULL, ...) {
 
 #' Simulate from a communityPGLMM object
 #'
-#' Note that this function currently only works for model fit with \code{bayes = TRUE}
-#'
 #' @inheritParams lme4::simulate.merMod
 #' @param re.form (formula, `NULL`, or `NA`) specify which random effects to condition on when predicting. 
 #' If `NULL`, include all random effects and the conditional modes of those random effects will be included in the deterministic part of the simulation (i.e Xb + Zu); 
 #' if `NA` or `~0`, include no random effects and new values will be chosen for each group based on the estimated random-effects variances (i.e. Xb + Zu * u_random).
 #' @param object A fitted model object with class 'communityPGLMM'.
+#' @param ntry Number of times to retry simulation in the case of `NA` values. Only applies
+#' to models fit with `bayes = TRUE`. If there are still `NA`s after `ntry` times, the 
+#' simulated values will be returned (with `NA`s) with a warning. If you keep getting `NA`s try
+#' rerunning with `full = TRUE`, which simulates in a slower but more stable way.
+#' @param full If `TRUE`, and the model was fit using `bayes = TRUE`, then the simulation will be done 
+#' with an approximation of the full joint posterior (rather than the marginal posterior, 
+#' which is the default). This method is much slower but is often more stable, and is 
+#' technically more accurate.
 #'
 #' @export
 #'
 simulate.communityPGLMM <- function(object, nsim = 1, seed = NULL, 
-                                    re.form = NULL, ...) {
+                                    re.form = NULL, ntry = 5, full = FALSE, ...) {
   if(!is.null(seed)) set.seed(seed)
   
   #sim <- INLA::inla.posterior.sample(nsim, object$inla.model)
@@ -1383,27 +1407,167 @@ simulate.communityPGLMM <- function(object, nsim = 1, seed = NULL,
       sim <- apply(mu_sim, MARGIN = 2, FUN = function(x) rbinom(length(x), Ntrials, x))
     }
   } else { # beyes version
-    if(deparse(re.form) == "~0" | deparse(re.form) == "NA")
-      warning("re.form = NULL is the only option for bayes models at this moment",
-              immediate. = TRUE)
-    mu_sim <- do.call(rbind, lapply(object$inla.model$marginals.fitted.values, 
-                                    INLA::inla.rmarginal, n = nsim)) %>%
-      as.data.frame()
     
-    if(object$bayes && object$family == "binomial" && 
-       !is.null(object$inla.model$Ntrials)) {
-      Ntrials <- object$inla.model$Ntrials
-    } else {
-      Ntrials <- 1
-    }
+    sim <- bayes_simulate(object, nsim = nsim, re.form = re.form, ntry = ntry, full = full)
     
-    sim <- switch(object$family,
-                  binomial = lapply(mu_sim, function(x) rbinom(length(x), Ntrials, x)),
-                  poisson = lapply(mu_sim, function(x) rpois(length(x), x))
-    )
-    
-    sim <- do.call(cbind, sim)
   }
   
   sim
 }
+
+bayes_simulate <- function(object, nsim = 250, re.form, ntry = 5, full = FALSE) {
+  
+  if(object$bayes && object$family == "binomial" && 
+     !is.null(object$inla.model$Ntrials)) {
+    Ntrials <- object$inla.model$Ntrials
+  } else {
+    Ntrials <- 1
+  }
+  
+  if(is.null(re.form)){
+    if(full) {
+      
+      fam <- family(object)
+      
+      mu_sim <- INLA::inla.posterior.sample(nsim, object$inla.model,
+                                            selection = list(Predictor = 0))
+      mu_sim <- lapply(mu_sim, function(x) fam$linkinv(x$latent))
+      
+      sim <- switch(object$family,
+                    binomial = lapply(mu_sim, function(x) rbinom(length(x), Ntrials, x)),
+                    poisson = lapply(mu_sim, function(x) rpois(length(x), x)),
+                    mu_sim
+      )
+      
+      sim <- do.call(cbind, sim) 
+      
+    } else {
+      
+      mu_sim <- do.call(rbind, lapply(object$inla.model$marginals.fitted.values, 
+                                    INLA::inla.rmarginal, n = nsim)) %>%
+        as.data.frame()
+      
+      sim <- switch(object$family,
+                    binomial = lapply(mu_sim, function(x) rbinom(length(x), Ntrials, x)),
+                    poisson = lapply(mu_sim, function(x) rpois(length(x), x)),
+                    mu_sim
+      )
+      
+      missing <- sapply(sim, function(x) any(is.na(x)))
+      
+      if(any(missing) && ntry > 0) {
+        for(i in seq_len(ntry)) {
+          mu_sim[missing] <- do.call(rbind, lapply(object$inla.model$marginals.fitted.values, 
+                                      INLA::inla.rmarginal, n = sum(missing))) %>%
+            as.data.frame()
+          sim[missing] <- switch(object$family,
+                                 binomial = lapply(mu_sim[missing], function(x) rbinom(length(x), Ntrials, x)),
+                                 poisson = lapply(mu_sim[missing], function(x) rpois(length(x), x)),
+                                 mu_sim
+          )
+          missing <- sapply(sim, function(x) any(is.na(x)))
+          if(!any(missing)) {
+            break
+          }
+        }
+      }
+      
+      if(any(missing)) {
+        warning("Simulation resulted in some NA values after ", ntry, " tries. Consider increasing ntry or using full = TRUE")
+      }
+      
+      sim <- do.call(cbind, sim)  
+      
+    }
+    
+    
+  } else {
+    
+    if(deparse(re.form) == "~0" | deparse(re.form) == "NA"){
+        
+      # condition on none of the random effects
+      if(full) {
+        
+        fam <- family(object)
+        
+        beta_names <- rownames(object$inla.model$summary.fixed)
+        selection <- as.list(rep(0, length(beta_names)))
+        names(selection) <- beta_names
+        beta_sim <- INLA::inla.posterior.sample(nsim, object$inla.model,
+                                                selection = selection)
+        
+        beta_sim <- sapply(beta_sim, function(x) fam$linkinv(x$latent))
+        
+        mu_sim <- (object$X %*% beta_sim) %>%
+          as.data.frame()
+        
+        mu_sim <- lapply(mu_sim, fam$linkinv)
+
+        sim <- switch(object$family,
+                      binomial = lapply(mu_sim, function(x) rbinom(length(x), Ntrials, x)),
+                      poisson = lapply(mu_sim, function(x) rpois(length(x), x)),
+                      mu_sim
+                      )
+        
+        sim <- do.call(cbind, sim) 
+        
+      } else {
+        
+        fam <- family(object)
+        
+        beta_sim <- do.call(cbind, lapply(object$inla.model$marginals.fixed,
+                                          INLA::inla.rmarginal, n = nsim)) 
+        
+        mu_sim <- (object$X %*% t(beta_sim)) %>%
+          as.data.frame()
+        
+        mu_sim <- lapply(mu_sim, fam$linkinv)
+
+        sim <- switch(object$family,
+                      binomial = lapply(mu_sim, function(x) rbinom(length(x), Ntrials, x)),
+                      poisson = lapply(mu_sim, function(x) rpois(length(x), x)),
+                      mu_sim
+                      )
+        
+        missing <- sapply(sim, function(x) any(is.na(x)))
+      
+        if(any(missing) && ntry > 0) {
+          for(i in seq_len(ntry)) {
+            beta_sim[missing, ] <- do.call(cbind, lapply(object$inla.model$marginals.fixed,
+                                                         INLA::inla.rmarginal, n = sum(missing))) 
+            mu_sim[missing] <- (object$X %*% t(beta_sim[missing, , drop = FALSE])) %>%
+              as.data.frame() %>%
+              lapply(fam$linkinv)
+            
+            sim[missing] <- switch(object$family,
+                                   binomial = lapply(mu_sim[missing], function(x) rbinom(length(x), Ntrials, x)),
+                                   poisson = lapply(mu_sim[missing], function(x) rpois(length(x), x)),
+                                   mu_sim
+            )
+            missing <- sapply(sim, function(x) any(is.na(x)))
+            if(!any(missing)) {
+              break
+            }
+          }
+        }
+          
+        if(any(missing)) {
+          warning("Simulation resulted in some NA values after ", ntry, " tries. Consider increasing ntry or using full = TRUE")
+        }
+          
+        sim <- do.call(cbind, sim) 
+        
+      }
+        
+        
+    } else {
+        
+      stop("Formula for random effects to condition on currently is not supported yet")
+      
+    }
+    
+  }
+  
+  sim
+  
+}
