@@ -1,5 +1,10 @@
 // -*- mode: C++; c-indent-level: 4; c-basic-offset: 4; indent-tabs-mode: nil; -*-
 
+/*
+ Prevents the warning "solve(): system is singular; attempting approx solution"
+ */
+#define ARMA_WARN_LEVEL 1
+
 #include <RcppArmadillo.h>
 #include <numeric>
 #include <cmath>
@@ -18,43 +23,35 @@ using namespace Rcpp;
 
 
 /*
- ***************************************************************************************
- ***************************************************************************************
- ***************************************************************************************
- ***************************************************************************************
+ *******************************************************************************
+ *******************************************************************************
+ *******************************************************************************
+ *******************************************************************************
  
  Log likelihood function
  
- ***************************************************************************************
- ***************************************************************************************
- ***************************************************************************************
- ***************************************************************************************
+ *******************************************************************************
+ *******************************************************************************
+ *******************************************************************************
+ *******************************************************************************
  */
 
 
-
-
-
 // `cor_phylo` log likelihood function.
 // 
 //[[Rcpp::export]]
 double cor_phylo_LL(NumericVector par,
-                    SEXP xptr) {
-  
-  XPtr<LogLikInfo> lli(xptr);
-  
-  const arma::mat& XX(lli->XX);
-  const arma::mat& UU(lli->UU);
-  const arma::mat& MM(lli->MM);
-  const arma::mat& Vphy(lli->Vphy);
-  const arma::mat& tau(lli->tau);
-  const bool& REML(lli->REML);
-  const bool& constrain_d(lli->constrain_d);
-  const double& lower_d(lli->lower_d);
-  const bool& verbose(lli->verbose);
-  const double& rcond_threshold(lli->rcond_threshold);
-  
-  
+                    const arma::mat& XX,
+                    const arma::mat& UU,
+                    const arma::mat& MM,
+                    const arma::mat& Vphy,
+                    const arma::mat& tau,
+                    const bool& REML,
+                    const bool& constrain_d,
+                    const double& lower_d,
+                    const bool& verbose,
+                    const double& rcond_threshold) {
+
   bool return_max = false;
   
   uint_t n = Vphy.n_rows;
@@ -71,14 +68,16 @@ double cor_phylo_LL(NumericVector par,
   arma::mat C = make_C(n, p, tau, d, Vphy, R);
   
   arma::mat V = make_V(C, MM);
+  if (V.has_nan() || V.has_inf()) return MAX_RETURN;
   double rcond_dbl = 0;
   rcond_dbl = arma::rcond(V);
-  if (!arma::is_finite(rcond_dbl) || rcond_dbl < rcond_threshold) return MAX_RETURN;
+  if (!std::isfinite(rcond_dbl) || rcond_dbl < rcond_threshold) return MAX_RETURN;
   
   arma::mat iV = arma::inv(V);
   arma::mat denom = UU.t() * iV * UU;
+  if (denom.has_nan() || denom.has_inf()) return MAX_RETURN;
   rcond_dbl = arma::rcond(denom);
-  if (!arma::is_finite(rcond_dbl) || rcond_dbl < rcond_threshold) return MAX_RETURN;
+  if (!std::isfinite(rcond_dbl) || rcond_dbl < rcond_threshold) return MAX_RETURN;
   
   arma::mat num = UU.t() * iV * XX;
   arma::vec B0 = arma::solve(denom, num);
@@ -86,7 +85,7 @@ double cor_phylo_LL(NumericVector par,
   
   double logdetV, det_sign;
   arma::log_det(logdetV, det_sign, iV);
-  if (!arma::is_finite(logdetV)) return MAX_RETURN;
+  if (!std::isfinite(logdetV)) return MAX_RETURN;
   logdetV *= -1;
   
   double LL;
@@ -112,6 +111,9 @@ double cor_phylo_LL(NumericVector par,
 
 
 
+
+
+
 /*
  Return reciprocal condition numbers for matrices in the log likelihood function.
  
@@ -119,19 +121,19 @@ double cor_phylo_LL(NumericVector par,
  It is used in the output to guide users wanting to change the `rcond_threshold`
  argument.
  */
-std::vector<double> return_rcond_vals(XPtr<LogLikInfo> ll_info) {
+std::vector<double> return_rcond_vals(const LogLikInfo& ll_info) {
   
-  const arma::vec& par(ll_info->min_par);
-  const arma::mat& XX(ll_info->XX);
-  const arma::mat& UU(ll_info->UU);
-  const arma::mat& MM(ll_info->MM);
-  const arma::mat& Vphy(ll_info->Vphy);
-  const arma::mat& tau(ll_info->tau);
-  // const bool& REML(ll_info->REML);
-  const bool& constrain_d(ll_info->constrain_d);
-  const double& lower_d(ll_info->lower_d);
-  // const bool& verbose(ll_info->verbose);
-  // const double& rcond_threshold(ll_info->rcond_threshold);
+  const arma::vec& par(ll_info.min_par);
+  const arma::mat& XX(ll_info.XX);
+  const arma::mat& UU(ll_info.UU);
+  const arma::mat& MM(ll_info.MM);
+  const arma::mat& Vphy(ll_info.Vphy);
+  const arma::mat& tau(ll_info.tau);
+  // const bool& REML(ll_info.REML);
+  const bool& constrain_d(ll_info.constrain_d);
+  const double& lower_d(ll_info.lower_d);
+  // const bool& verbose(ll_info.verbose);
+  // const double& rcond_threshold(ll_info.rcond_threshold);
   
   std::vector<double> rconds_out(2);
   
@@ -149,12 +151,20 @@ std::vector<double> return_rcond_vals(XPtr<LogLikInfo> ll_info) {
   
   arma::mat V = make_V(C, MM);
   double rcond_dbl = 0;
-  rcond_dbl = arma::rcond(V);
+  if (V.has_nan() || V.has_inf()) {
+    rcond_dbl = arma::datum::nan;
+  } else {
+    rcond_dbl = arma::rcond(V);
+  }
   rconds_out[0] = rcond_dbl;
   
   arma::mat iV = arma::inv(V);
   arma::mat denom = UU.t() * iV * UU;
-  rcond_dbl = arma::rcond(denom);
+  if (denom.has_nan() || denom.has_inf()) {
+    rcond_dbl = arma::datum::nan;
+  } else {
+    rcond_dbl = arma::rcond(denom);
+  }
   rconds_out[1] = rcond_dbl;
   
   return rconds_out;
@@ -167,23 +177,23 @@ std::vector<double> return_rcond_vals(XPtr<LogLikInfo> ll_info) {
 
 
 /*
- ***************************************************************************************
- ***************************************************************************************
- ***************************************************************************************
- ***************************************************************************************
+ *******************************************************************************
+ *******************************************************************************
+ *******************************************************************************
+ *******************************************************************************
  
  Fit using nlopt
  
- ***************************************************************************************
- ***************************************************************************************
- ***************************************************************************************
- ***************************************************************************************
+ *******************************************************************************
+ *******************************************************************************
+ *******************************************************************************
+ *******************************************************************************
  */
 
 /*
  Fit cor_phylo model using nlopt.
  */
-void fit_cor_phylo_nlopt(XPtr<LogLikInfo> ll_info,
+void fit_cor_phylo_nlopt(LogLikInfo& ll_info,
                          const double& rel_tol,
                          const int& max_iter,
                          const std::string& method) {
@@ -206,33 +216,42 @@ void fit_cor_phylo_nlopt(XPtr<LogLikInfo> ll_info,
                               _["xtol_rel"] = 0.0001,
                               _["maxeval"] = max_iter);
   
-  NumericVector par0(ll_info->par0.begin(), ll_info->par0.end());
+  NumericVector par0(ll_info.par0.begin(), ll_info.par0.end());
   
   List opt = nloptr(_["x0"] = par0,
                    _["eval_f"] = cor_phylo_LL_fxn,
                    _["opts"] = options,
-                   _["xptr"] = Rcpp::wrap(ll_info));
+                   _["XX"] = ll_info.XX,
+                   _["UU"] = ll_info.UU,
+                   _["MM"] = ll_info.MM,
+                   _["Vphy"] = ll_info.Vphy,
+                   _["tau"] = ll_info.tau,
+                   _["REML"] = ll_info.REML,
+                   _["constrain_d"] = ll_info.constrain_d,
+                   _["lower_d"] = ll_info.lower_d,
+                   _["verbose"] = ll_info.verbose,
+                   _["rcond_threshold"] = ll_info.rcond_threshold);
   
-  ll_info->min_par = as<arma::vec>(opt["solution"]);
+  ll_info.min_par = as<arma::vec>(opt["solution"]);
   
-  ll_info->LL = as<double>(opt["objective"]);
+  ll_info.LL = as<double>(opt["objective"]);
   int convcode_ = as<int>(opt["status"]);
   
   if (convcode_ > 0) {
     if (convcode_ < 5) {
-      ll_info->convcode = 0;
+      ll_info.convcode = 0;
     } else {
-      ll_info->convcode = 1;
+      ll_info.convcode = 1;
     }
   } else {
-    ll_info->convcode = -1 * convcode_ + 1;
+    ll_info.convcode = -1 * convcode_ + 1;
   }
 
-  ll_info->iters = as<arma::vec>(opt["iterations"])(0);
+  ll_info.iters = as<arma::vec>(opt["iterations"])(0);
   
-  if (ll_info->verbose) {
-    Rcout << ll_info->LL << ' ';
-    arma::vec& par(ll_info->min_par);
+  if (ll_info.verbose) {
+    Rcout << ll_info.LL << ' ';
+    arma::vec& par(ll_info.min_par);
     for (uint_t i = 0; i < par.n_elem; i++) Rcout << par(i) << ' ';
     Rcout << std::endl;
   }
@@ -247,7 +266,7 @@ void fit_cor_phylo_nlopt(XPtr<LogLikInfo> ll_info,
  Make sure this doesn't get run in parallel!
 
  */
-void fit_cor_phylo_R(XPtr<LogLikInfo> ll_info,
+void fit_cor_phylo_R(LogLikInfo& ll_info,
                      const double& rel_tol,
                      const int& max_iter,
                      const std::string& method,
@@ -261,36 +280,59 @@ void fit_cor_phylo_R(XPtr<LogLikInfo> ll_info,
   
   Rcpp::List opt;
 
-  NumericVector par0(ll_info->par0.begin(), ll_info->par0.end());
+  NumericVector par0(ll_info.par0.begin(), ll_info.par0.end());
   
   if (method == "sann") {
+    List sann_control;
+    sann_control["maxit"] = sann[0];
+    sann_control["temp"] = sann[1];
+    sann_control["tmax"] = sann[2];
+    sann_control["reltol"] = rel_tol;
     opt = optim(_["par"] = par0,
                 _["fn"] = cor_phylo_LL_fxn,
                 _["method"] = "SANN",
-                _["control"] = List::create(_["maxit"] = sann[0],
-                                            _["temp"] = sann[1],
-                                            _["tmax"] = sann[2],
-                                            _["reltol"] = rel_tol),
-                _["xptr"] = Rcpp::wrap(ll_info));
+                _["control"] = sann_control,
+                _["XX"] = ll_info.XX,
+                _["UU"] = ll_info.UU,
+                _["MM"] = ll_info.MM,
+                _["Vphy"] = ll_info.Vphy,
+                _["tau"] = ll_info.tau,
+                _["REML"] = ll_info.REML,
+                _["constrain_d"] = ll_info.constrain_d,
+                _["lower_d"] = ll_info.lower_d,
+                _["verbose"] = ll_info.verbose,
+                _["rcond_threshold"] = ll_info.rcond_threshold);
     par0 = as<NumericVector>(opt["par"]);
   }
   
+  List optim_control = List::create(_["maxit"] = max_iter, 
+                                    _["reltol"] = rel_tol);
+  
   opt = optim(_["par"] = par0,
               _["fn"] = cor_phylo_LL_fxn,
               _["method"] = "Nelder-Mead",
-              _["control"] = List::create(_["maxit"] = max_iter,
-                                          _["reltol"] = rel_tol),
-              _["xptr"] = Rcpp::wrap(ll_info));
+              _["control"] = optim_control,
+              _["XX"] = ll_info.XX,
+              _["UU"] = ll_info.UU,
+              _["MM"] = ll_info.MM,
+              _["Vphy"] = ll_info.Vphy,
+              _["tau"] = ll_info.tau,
+              _["REML"] = ll_info.REML,
+              _["constrain_d"] = ll_info.constrain_d,
+              _["lower_d"] = ll_info.lower_d,
+              _["verbose"] = ll_info.verbose,
+              _["rcond_threshold"] = ll_info.rcond_threshold);
   
-  ll_info->min_par = as<arma::vec>(opt["par"]);
   
-  ll_info->LL = as<double>(opt["value"]);
-  ll_info->convcode = as<int>(opt["convergence"]);
-  ll_info->iters = as<arma::vec>(opt["counts"])(0);
+  ll_info.min_par = as<arma::vec>(opt["par"]);
   
-  if (ll_info->verbose) {
-    Rcout << ll_info->LL << ' ';
-    const arma::vec& par(ll_info->min_par);
+  ll_info.LL = as<double>(opt["value"]);
+  ll_info.convcode = as<int>(opt["convergence"]);
+  ll_info.iters = as<arma::vec>(opt["counts"])(0);
+  
+  if (ll_info.verbose) {
+    Rcout << ll_info.LL << ' ';
+    const arma::vec& par(ll_info.min_par);
     for (uint_t i = 0; i < par.n_elem; i++) Rcout << par(i) << ' ';
     Rcout << std::endl;
   }
@@ -302,17 +344,17 @@ void fit_cor_phylo_R(XPtr<LogLikInfo> ll_info,
 
 
 /*
- ***************************************************************************************
- ***************************************************************************************
- ***************************************************************************************
- ***************************************************************************************
+ *******************************************************************************
+ *******************************************************************************
+ *******************************************************************************
+ *******************************************************************************
  
  Other functions
  
- ***************************************************************************************
- ***************************************************************************************
- ***************************************************************************************
- ***************************************************************************************
+ *******************************************************************************
+ *******************************************************************************
+ *******************************************************************************
+ *******************************************************************************
  */
 
 /*-
@@ -451,10 +493,10 @@ LogLikInfo::LogLikInfo(const arma::mat& X,
 LogLikInfo::LogLikInfo(const arma::mat& X,
                  const std::vector<arma::mat>& U,
                  const arma::mat& M,
-                 XPtr<LogLikInfo> other) 
-  : UU(other->UU), Vphy(other->Vphy), tau(other->tau), REML(other->REML),
-    no_corr(other->no_corr), constrain_d(other->constrain_d), lower_d(other->lower_d),
-    verbose(other->verbose), rcond_threshold(other->rcond_threshold), iters(0) {
+                 const LogLikInfo& other) 
+  : UU(other.UU), Vphy(other.Vphy), tau(other.tau), REML(other.REML),
+    no_corr(other.no_corr), constrain_d(other.constrain_d), lower_d(other.lower_d),
+    verbose(other.verbose), rcond_threshold(other.rcond_threshold), iters(0) {
 
   uint_t p = X.n_cols;
   
@@ -494,34 +536,34 @@ LogLikInfo::LogLikInfo(const arma::mat& X,
 
 
 inline void main_output(arma::mat& corrs, arma::mat& B, arma::mat& B_cov, arma::vec& d,
-                        XPtr<LogLikInfo> ll_info,
+                        const LogLikInfo& ll_info,
                         const arma::mat& X, const std::vector<arma::mat>& U) {
   
   uint_t n = X.n_rows;
   uint_t p = X.n_cols;
   
-  arma::mat L = make_L(ll_info->min_par, p);
+  arma::mat L = make_L(ll_info.min_par, p);
   
   arma::mat R = L.t() * L;
   
   corrs = make_corrs(R);
   
-  d = make_d(ll_info->min_par, p, ll_info->constrain_d, ll_info->lower_d);
+  d = make_d(ll_info.min_par, p, ll_info.constrain_d, ll_info.lower_d);
   
   // OU transform
-  arma::mat C = make_C(n, p, ll_info->tau, d, ll_info->Vphy, R);
+  arma::mat C = make_C(n, p, ll_info.tau, d, ll_info.Vphy, R);
   
-  arma::mat V = make_V(C, ll_info->MM);
+  arma::mat V = make_V(C, ll_info.MM);
   
   arma::mat iV = arma::inv(V);
   
-  arma::mat denom = ll_info->UU.t() * iV * ll_info->UU;
+  arma::mat denom = ll_info.UU.t() * iV * ll_info.UU;
   
-  arma::mat num = ll_info->UU.t() * iV * ll_info->XX;
+  arma::mat num = ll_info.UU.t() * iV * ll_info.XX;
   
   arma::vec B0 = arma::solve(denom, num);
   
-  make_B_B_cov(B, B_cov, B0, iV, ll_info->UU, X, U);
+  make_B_B_cov(B, B_cov, B0, iV, ll_info.UU, X, U);
   
   return;
 }
@@ -536,13 +578,7 @@ inline void main_output(arma::mat& corrs, arma::mat& B, arma::mat& B_cov, arma::
 List cp_get_output(const arma::mat& X,
                    const std::vector<arma::mat>& U,
                    const arma::mat& M,
-                   XPtr<LogLikInfo> ll_info,
-                   const double& rel_tol,
-                   const int& max_iter,
-                   const std::string& method,
-                   const uint_t& boot,
-                   const std::string& keep_boots,
-                   const std::vector<double>& sann) {
+                   const LogLikInfo& ll_info) {
 
   
   uint_t n = X.n_rows;
@@ -560,44 +596,24 @@ List cp_get_output(const arma::mat& X,
   main_output(corrs, B, B_cov, d, ll_info, X, U);
   
   double logLik = -0.5 * std::log(2 * arma::datum::pi);
-  if (ll_info->REML) {
-    logLik *= (n * p - ll_info->UU.n_cols);
-    arma::mat to_det = ll_info->XX.t() * ll_info->XX;
+  if (ll_info.REML) {
+    logLik *= (n * p - ll_info.UU.n_cols);
+    arma::mat to_det = ll_info.XX.t() * ll_info.XX;
     double det_val, det_sign;
     arma::log_det(det_val, det_sign, to_det);
-    logLik += 0.5 * det_val - ll_info->LL;
+    logLik += 0.5 * det_val - ll_info.LL;
   } else {
     logLik *= (n * p);
-    logLik -= ll_info->LL;
+    logLik -= ll_info.LL;
   }
   
-  double k = ll_info->min_par.n_elem + ll_info->UU.n_cols;
+  double k = ll_info.min_par.n_elem + ll_info.UU.n_cols;
   double AIC, BIC;
   AIC = -2 * logLik + 2 * k;
   BIC = -2 * logLik + k * std::log(n / arma::datum::pi);
   
   std::vector<double> rcond_vals = return_rcond_vals(ll_info);
   
-  List boot_list = List::create();
-  if (boot > 0) {
-    // `BootMats` stores matrices that we'll need for bootstrapping
-    BootMats bm(X, U, M, B, d, ll_info);
-    BootResults br(p, B.n_rows, boot);
-    for (uint_t b = 0; b < boot; b++) {
-      Rcpp::checkUserInterrupt();
-      bm.one_boot(ll_info, br, b, rel_tol, max_iter, method, keep_boots, sann);
-    }
-    std::vector<NumericMatrix> boot_out_mats(br.out_inds.size());
-    for (uint_t i = 0; i < br.out_inds.size(); i++) {
-      boot_out_mats[i] = wrap(br.out_mats[i]);
-    }
-    boot_list = List::create(_["corrs"] = br.corrs, _["d"] = br.d,
-                             _["B0"] = br.B0, _["B_cov"] = br.B_cov,
-                             _["inds"] = br.out_inds,
-                             _["convcodes"] = br.out_codes,
-                             _["mats"] = boot_out_mats);
-  }
-  
   // Now the final output list
   List out = List::create(
     _["corrs"] = corrs,
@@ -607,10 +623,11 @@ List cp_get_output(const arma::mat& X,
     _["logLik"] = logLik,
     _["AIC"] = AIC,
     _["BIC"] = BIC,
-    _["niter"] = ll_info->iters,
-    _["convcode"] = ll_info->convcode,
+    _["niter"] = ll_info.iters,
+    _["convcode"] = ll_info.convcode,
     _["rcond_vals"] = rcond_vals,
-    _["bootstrap"] = boot_list
+    _["min_par"] = ll_info.min_par,
+    _["bootstrap"] = List::create()
   );
   
   return out;
@@ -629,8 +646,8 @@ List cp_get_output(const arma::mat& X,
 //' @inheritParams cor_phylo
 //' @param method the `method` input to `cor_phylo`.
 //' 
-//' @return a list containing output information, to later be coerced to a `cor_phylo`
-//'   object by the `cor_phylo` function.
+//' @return a list containing output information, to later be coerced to a 
+//'   `cor_phylo` object by the `cor_phylo` function.
 //' @noRd
 //' @name cor_phylo_cpp
 //' 
@@ -638,7 +655,7 @@ List cp_get_output(const arma::mat& X,
 List cor_phylo_cpp(const arma::mat& X,
                    const std::vector<arma::mat>& U,
                    const arma::mat& M,
-                   const arma::mat& Vphy_,
+                   const arma::mat& Vphy,
                    const bool& REML,
                    const bool& constrain_d,
                    const double& lower_d,
@@ -648,14 +665,12 @@ List cor_phylo_cpp(const arma::mat& X,
                    const int& max_iter,
                    const std::string& method,
                    const bool& no_corr,
-                   const uint_fast32_t& boot,
-                   const std::string& keep_boots,
                    const std::vector<double>& sann) {
   
 
   // LogLikInfo is C++ class to use for organizing info for optimizing
-  XPtr<LogLikInfo> ll_info(new LogLikInfo(X, U, M, Vphy_, REML, no_corr, constrain_d, 
-                                          lower_d, verbose, rcond_threshold), true);
+  LogLikInfo ll_info(X, U, M, Vphy, REML, no_corr, constrain_d, 
+                     lower_d, verbose, rcond_threshold);
 
   /*
    Do the fitting.
@@ -670,115 +685,69 @@ List cor_phylo_cpp(const arma::mat& X,
   
   // Retrieve output from `ll_info` object and convert to list
   // Also do bootstrapping if desired
-  List output = cp_get_output(X, U, M, ll_info, rel_tol, max_iter, method,
-                              boot, keep_boots, sann);
+  List output = cp_get_output(X, U, M, ll_info);
   
   return output;
   
 }
 
-
-
-
-
-
-
-
-
-
-/*
- ***************************************************************************************
- ***************************************************************************************
- ***************************************************************************************
- ***************************************************************************************
- 
- Bootstrapping functions
- 
- ***************************************************************************************
- ***************************************************************************************
- ***************************************************************************************
- ***************************************************************************************
- */
-
-
-
-BootMats::BootMats(const arma::mat& X_, 
-                   const std::vector<arma::mat>& U_,
-                   const arma::mat& M_,
-                   const arma::mat& B_, 
-                   const arma::vec& d_, 
-                   XPtr<LogLikInfo> ll_info)
-  : X(X_), U(U_), M(M_), X_new(), iD(), X_pred() {
+//' Inner function to do one bootstrapping rep.
+//' 
+//' The arguments before `X` are needed in this function but not in
+//' `cor_phylo_cpp`, while the arguments starting at `X` are all those required
+//' in `cor_phylo_cpp`.
+//'
+//' @noRd
+//' 
+//[[Rcpp::export]]
+List one_boot_cpp(const arma::vec& rnd_vec,
+                  const arma::vec& min_par,
+                  const arma::mat& B,
+                  const arma::vec& d,
+                  const std::string& keep_boots,
+                  const arma::mat& X,
+                  const std::vector<arma::mat>& U,
+                  const arma::mat& M,
+                  const arma::mat& Vphy,
+                  const bool& REML,
+                  const bool& constrain_d,
+                  const double& lower_d,
+                  const bool& verbose,
+                  const double& rcond_threshold,
+                  const double& rel_tol,
+                  const int& max_iter,
+                  const std::string& method,
+                  const bool& no_corr,
+                  const std::vector<double>& sann) {
   
-  uint_t n = ll_info->Vphy.n_rows;
-  uint_t p = X.n_cols;
-  
-  arma::mat L = make_L(ll_info->min_par, p);
-  arma::mat R = L.t() * L;
-  arma::mat C = make_C(n, p, ll_info->tau, d_, ll_info->Vphy, R);
-  arma::mat V = make_V(C, ll_info->MM);
-  iD = V;
-  safe_chol(iD, "bootstrapping-matrices setup");
-  iD = iD.t();
-  
-  // For predicted X values (i.e., without error)
-  X_pred = ll_info->UU.t();
-  arma::rowvec tmp = B_.col(0).t();
-  X_pred = tmp * X_pred;
-  X_pred = X_pred.t();
-  X_pred.reshape(n, p);
-  
-  return;
-}
-
-
-/*
- Iterate from a BootMats object in prep for a bootstrap replicate.
- 
- This ultimately updates the LogLikInfo object with new XX and MM matrices,
- and updates the BootResults object with the mean and sd.
- */
-XPtr<LogLikInfo> BootMats::iterate(XPtr<LogLikInfo> ll_info) {
-
   uint_t n = X.n_rows;
   uint_t p = X.n_cols;
   
-  X_new = X_pred;
+  LogLikInfo ll_info(X, U, M, Vphy, REML, no_corr, constrain_d, 
+                     lower_d, verbose, rcond_threshold);
+  ll_info.min_par = min_par;
   
-  arma::mat X_rnd = iD * as<arma::vec>(rnorm(n * p));
-  X_rnd.reshape(n, p);
+  // Matrix to simulate error with phylogenetic signal
+  arma::mat iD = make_iD(ll_info, p, d);
   
+  // For predicted X values (i.e., without error)
+  arma::mat pred_X = ll_info.UU.t();
+  arma::rowvec tmp = B.col(0).t();
+  pred_X = tmp * pred_X;
+  pred_X = pred_X.t();
+  pred_X.reshape(n, p);
+  
+  // Add error with phylogenetic signal (rnd_vec should be ~ N(0,1)):
+  arma::mat rnd_X = iD * rnd_vec;
+  rnd_X.reshape(n, p);
   for (uint_t i = 0; i < p; i++) {
     double sd_ = arma::stddev(X.col(i));
-    X_new.col(i) += (X_rnd.col(i) * sd_);
+    rnd_X.col(i) *= sd_;
   }
-  // X_new = X_pred + X_rnd;
-
-  XPtr<LogLikInfo> ll_info_new(new LogLikInfo(X_new, U, M, ll_info), true);
-
-  return ll_info_new;
-}
-
-
-
-// Method to return bootstrapped data
-void BootMats::boot_data(XPtr<LogLikInfo> ll_info, BootResults& br, const uint_t& i) {
+  arma::mat new_X = pred_X + rnd_X;
   
-  br.out_inds.push_back(i+1);
-  br.out_codes.push_back(ll_info->convcode);
-  
-  br.out_mats.push_back(X_new);
-  
-  return;
-}
-
-void BootMats::one_boot(XPtr<LogLikInfo> ll_info, BootResults& br,
-                        const uint_t& i, const double& rel_tol, const int& max_iter,
-                        const std::string& method, const std::string& keep_boots,
-                        const std::vector<double>& sann) {
-  
-  // Generate new data
-  XPtr<LogLikInfo> new_ll_info = iterate(ll_info);
+  // Object to store info for new fit
+  LogLikInfo ll_info_new(new_X, U, M, ll_info);
   
   /*
    Do the fitting.
@@ -787,27 +756,94 @@ void BootMats::one_boot(XPtr<LogLikInfo> ll_info, BootResults& br,
    */
   if (method == "nelder-mead-r" || method == "sann") {
     // Do the fitting:
-    fit_cor_phylo_R(new_ll_info, rel_tol, max_iter, method, sann);
+    fit_cor_phylo_R(ll_info_new, rel_tol, max_iter, method, sann);
   } else {
     // Same thing for nlopt version
-    fit_cor_phylo_nlopt(new_ll_info, rel_tol, max_iter, method);
+    fit_cor_phylo_nlopt(ll_info_new, rel_tol, max_iter, method);
   }
-  // Determine whether convergence failed:
-  bool failed = new_ll_info->convcode != 0;
   
+  // Primary output objects:
+  arma::mat new_corrs;
+  arma::mat new_B;
+  arma::mat new_B_cov;
+  arma::vec new_d;
+  main_output(new_corrs, new_B, new_B_cov, new_d, ll_info_new, new_X, U);
+  
+  List boot_list = List::create(_["corrs"] = new_corrs, 
+                                _["d"] = new_d,
+                                _["B0"] = new_B.col(0), 
+                                _["B_cov"] = new_B_cov,
+                                _["convcodes"] = ll_info_new.convcode);
+  
+  // Determine whether convergence failed, and save simulated data if specified
+  bool failed = ll_info_new.convcode != 0;
   if (keep_boots == "all" || (keep_boots == "fail" && failed)) {
-    boot_data(new_ll_info, br, i);
+    boot_list["mats"] = new_X;
+  } else if (keep_boots == "fail" && !failed) {
+    // Add filler matrix if only keeping failed reps and this one succeeded
+    boot_list["mats"] = arma::mat(0,0);
   }
-
-  arma::mat corrs;
-  arma::mat B;
-  arma::mat B_cov;
-  arma::vec d;
-  main_output(corrs, B, B_cov, d, new_ll_info, X_new, U);
-
-  // Add values to BootResults
-  br.insert_values(i, corrs, B.col(0), B_cov, d);
   
-  return;
+  return boot_list;
+  
+}
+
+
+// Organize bootstrap output.
+//[[Rcpp::export]]
+List organize_boots(List orig_list) {
+  
+  uint_t n_boots = orig_list.size();
+  
+  // Get some basic info from the first bootstrap
+  List front_boot(orig_list[0]);
+  bool has_mats = front_boot.containsElementNamed("mats");
+  uint_t p = as<NumericMatrix>(front_boot["corrs"]).nrow();
+  uint_t B_rows = as<NumericVector>(front_boot["B0"]).size();
+  
+  arma::cube corrs(p, p, n_boots);
+  arma::mat d(p, n_boots);
+  arma::mat B0(B_rows, n_boots);
+  arma::cube B_cov(B_rows, B_rows, n_boots);
+  arma::Col<int> convcodes(n_boots);
+  List mats;
+  if (has_mats) mats = List(n_boots);
+  
+  arma::mat corrs_i;
+  arma::vec B0_i;
+  arma::mat B_cov_i;
+  arma::vec d_i;
+  NumericMatrix mats_i;
+  
+  for (uint_t i = 0; i < n_boots; i++) {
+    
+    List boot_i(orig_list[i]);
+    
+    corrs_i = as<arma::mat>(boot_i["corrs"]);
+    B0_i = as<arma::vec>(boot_i["B0"]);
+    B_cov_i = as<arma::mat>(boot_i["B_cov"]);
+    d_i = as<arma::vec>(boot_i["d"]);
+    
+    corrs.slice(i) = corrs_i;
+    B0.col(i) = B0_i;
+    B_cov.slice(i) = B_cov_i;
+    d.col(i) = d_i;
+    convcodes(i) = as<int>(boot_i["convcodes"]);
+    
+    if (has_mats) {
+      mats_i = as<NumericMatrix>(boot_i["mats"]);
+      mats(i) = mats_i;
+    }
+    
+  }
+  
+  List new_list = List::create(_["corrs"] = corrs,
+                               _["d"] = d,
+                               _["B0"] = B0, 
+                               _["B_cov"] = B_cov,
+                               _["convcodes"] = convcodes,
+                               _["mats"] = mats);
+  
+  return new_list;
 }
 
