@@ -32,6 +32,8 @@ using namespace Rcpp;
 
 
 
+
+
 // Info to calculate the log-likelihood
 class LogLikInfo {
 public:
@@ -67,7 +69,7 @@ public:
   LogLikInfo(const arma::mat& X,
           const std::vector<arma::mat>& U,
           const arma::mat& M,
-          XPtr<LogLikInfo> other);
+          const LogLikInfo& other);
   // Copy constructor
   LogLikInfo(const LogLikInfo& ll_info2) {
     par0 = ll_info2.par0;
@@ -92,76 +94,6 @@ public:
 
 
 
-// Results from bootstrapping
-
-class BootResults {
-public:
-  arma::cube corrs;
-  arma::mat B0;
-  arma::cube B_cov;
-  arma::mat d;
-  std::vector<arma::mat> out_mats;
-  std::vector<uint_t> out_inds;
-  std::vector<int> out_codes;
-
-  BootResults(const uint_t& p, const uint_t& B_rows, const uint_t& n_reps) 
-    : corrs(p, p, n_reps, arma::fill::zeros), 
-      B0(B_rows, n_reps, arma::fill::zeros), 
-      B_cov(B_rows, B_rows, n_reps, arma::fill::zeros),
-      d(p, n_reps, arma::fill::zeros), 
-      out_mats(), out_inds(), out_codes() {};
-
-  // Insert values into a BootResults object
-  void insert_values(const uint_t& i,
-                     const arma::mat& corrs_i,
-                     const arma::vec& B0_i,
-                     const arma::mat& B_cov_i,
-                     const arma::vec& d_i) {
-    
-    corrs.slice(i) = corrs_i;
-    B0.col(i) = B0_i;
-    B_cov.slice(i) = B_cov_i;
-    d.col(i) = d_i;
-    return;
-    
-  }
-  
-};
-
-
-/*
- Matrices to be kept for bootstrapping
- One per core if doing multi-threaded
- */
-class BootMats {
-public:
-  // original input matrices
-  const arma::mat X;
-  const std::vector<arma::mat> U;
-  const arma::mat M;
-  arma::mat X_new;
-  
-  BootMats(const arma::mat& X_, const std::vector<arma::mat>& U_,
-            const arma::mat& M_,
-            const arma::mat& B_, const arma::vec& d_, XPtr<LogLikInfo> ll_info);
-  
-  XPtr<LogLikInfo> iterate(XPtr<LogLikInfo> ll_info);
-  
-  void one_boot(XPtr<LogLikInfo> ll_info, BootResults& br,
-                const uint_t& i, const double& rel_tol, const int& max_iter,
-                const std::string& method, const std::string& keep_boots,
-                const std::vector<double>& sann);
-  
-  
-private:
-  arma::mat iD;
-  arma::mat X_pred;
-
-  // Method for returning bootstrapped data
-  void boot_data(XPtr<LogLikInfo> ll_info, BootResults& br, const uint_t& i);
-
-};
-
 
 
 
@@ -181,6 +113,36 @@ private:
  ***************************************************************************************
  */
 
+
+
+/*
+ Returning useful error message if choleski decomposition fails:
+ */
+inline void safe_chol(arma::mat& L, std::string task) {
+  try {
+    L = arma::chol(L);
+  } catch(const std::runtime_error& re) {
+    std::string err_msg = static_cast<std::string>(re.what());
+    if (err_msg == "chol(): decomposition failed") {
+      std::string err_msg_out = "Choleski decomposition failed during " + task + ". ";
+      err_msg_out += "Changing the `constrain_d` argument to `TRUE`, and ";
+      err_msg_out += "using a different algorithm (`method` argument) can remedy this.";
+      throw(Rcpp::exception(err_msg_out.c_str(), false));
+    } else {
+      std::string err_msg_out = "Runtime error: \n" + err_msg;
+      throw(Rcpp::exception(err_msg_out.c_str(), false));
+    }
+  } catch(const std::exception& ex) {
+    std::string err_msg = static_cast<std::string>(ex.what());
+    stop("Error occurred: \n" + err_msg);
+  } catch(...) {
+    stop("Unknown failure occurred.");
+  }
+  return;
+}
+
+
+
 // Flexible power function needed for multiple functions below
 // a^b
 
@@ -435,6 +397,33 @@ inline arma::mat make_corrs(const arma::mat& R) {
   return corrs;
 }
 
+inline arma::mat make_iD(const LogLikInfo& ll_info, 
+                         const uint_t& p, 
+                         const arma::vec& d_) {
+  uint_t n = ll_info.Vphy.n_rows;
+  arma::mat L = make_L(ll_info.min_par, p);
+  arma::mat R = L.t() * L;
+  arma::mat C = make_C(n, p, ll_info.tau, d_, ll_info.Vphy, R);
+  arma::mat V = make_V(C, ll_info.MM);
+  arma::mat iD = V;
+  safe_chol(iD, "bootstrapping-matrices setup");
+  iD = iD.t();
+  return iD;
+}
+
+
+// Standardize Vphy matrix
+inline arma::mat clean_Vphy(const arma::mat& Vphy_) {
+  double n = static_cast<double>(Vphy_.n_rows);
+  arma::mat Vphy = Vphy_;
+  Vphy /= Vphy_.max();
+  double val, sign;
+  arma::log_det(val, sign, Vphy);
+  val = std::exp(val / n);
+  Vphy /= val;
+  return Vphy;
+}
+
 
 /*
  Make matrices of coefficient estimates and standard errors, and matrix of covariances.
@@ -486,32 +475,6 @@ inline void make_B_B_cov(arma::mat& B, arma::mat& B_cov, arma::vec& B0,
 
 
 
-/*
- Returning useful error message if choleski decomposition fails:
- */
-inline void safe_chol(arma::mat& L, std::string task) {
-  try {
-    L = arma::chol(L);
-  } catch(const std::runtime_error& re) {
-    std::string err_msg = static_cast<std::string>(re.what());
-    if (err_msg == "chol(): decomposition failed") {
-      std::string err_msg_out = "Choleski decomposition failed during " + task + ". ";
-      err_msg_out += "Changing the `constrain_d` argument to `TRUE`, and ";
-      err_msg_out += "using a different algorithm (`method` argument) can remedy this.";
-      throw(Rcpp::exception(err_msg_out.c_str(), false));
-    } else {
-      std::string err_msg_out = "Runtime error: \n" + err_msg;
-      throw(Rcpp::exception(err_msg_out.c_str(), false));
-    }
-  } catch(const std::exception& ex) {
-    std::string err_msg = static_cast<std::string>(ex.what());
-    stop("Error occurred: \n" + err_msg);
-  } catch(...) {
-    stop("Unknown failure occurred.");
-  }
-  return;
-}
-
 
 
 #endif
